--[[
Special thanks to:

  * Bartender3 - Several portions of the Bartender3 source code were studied 
  	as a teaching aid for the development of this add-on.  In particular,
  	much of the layout and positioning code was based on Bartender3, with 
  	minor modifications to suit Numen's needs.  Many thanks and praises go 
  	to the Bartender3 Development Team for their work.
  	
  * AutoBar - The original concept of Numen was based around AutoBar.  While 
  	none of the code from AutoBar was used or consulted, without AutoBar, 
  	Numen may never have been created.
  	
  * Klappa - In combination with the WoW Forums, Klappa was used as a teaching 
  	aid for secure frames and state headers.  Many hours of understanding 
  	these components were spared because of Klappa.

  * WoW Forums - Many of the examples of secure frames and state headers found 
  	in the WoW Forums were used as a guide, in conjunction with Klappa, to 
  	create Numen.
  	
  * Ace2 and Libraries - This goes without saying.  Numen would not be here 
  	without the work of the Ace Development Team.  Many thanks goes out to 
  	them for creating a simple and easy framework to build add-ons on.
  
--]]


Numen = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "AceEvent-2.0", "FuBarPlugin-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Numen")
local Tablet = AceLibrary("Tablet-2.0")
local Gratuity = AceLibrary("Gratuity-2.0");

local _G = getfenv(0)

Numen.totems = {
	Earth = {
		localType = L["Earth"];
		index = 1,
		order = 1,
		spellCount = 0,
		spells = {
		},
	},
	Fire = {
		localType = L["Fire"];
		index = 2,
		order = 2,
		spellCount = 0,
		spells = {
		},
	},
	Water = {
		localType = L["Water"];
		index = 3,
		order = 3,
		spellCount = 0,
		spells = {
		},
	},
	Air = {
		localType = L["Air"];
		index = 4,
		order = 4,
		spellCount = 0,
		spells = {
		},
	},
	Weapon = {
		localType = L["Weapon"];
		index = 5,
		order = 5,
		spellCount = 0,
		spells = {
		},
	},
};

Numen.isLocked = true;

Numen.options = {
	type = "group",
	args = {
		bar = {
			type = "group",
			name = L["Bar Configuration"],
			desc = L["Configuration for the Numen Bar."],
			order = 1,
			args = {
				orient = {
					type = "text",
					name = L["Orientation"],
					desc = L["Set the orientation of the Numen Bar."],
					order = 1,
					get = function() return Numen.db.profile.orient; end,
					set = function(v) Numen.db.profile.orient = v; Numen:UpdateLayout(); end,
					validate = { 
						["horzup"] = L["Horizontal, Grow Up"],
						["horzdown"] = L["Horizontal, Grow Down"],
						["vertright"] = L["Vertical, Grow Right"],
						["vertleft"] = L["Vertical, Grow Left"],
					},
				},
				confScale = {
					type = "range",
					name = L["Main Scale"],
					desc = L["Set the scale of the main Numen buttons."],
					order = 4,
					step = 0.05,
					min = 0.25,
					max = 5.0,
					isPercent = false,
					get = function() return Numen.db.profile.confScale; end,
					set = function(v)
						Numen.db.profile.confScale = v;
						Numen.root:SetScale(v);
					end
				},
				confPopScale = {
					type = "range",
					name = L["PopUp Scale"],
					desc = L["Set the scale of the pop-up Numen buttons."],
					order = 6,
					step = 0.05,
					min = 0.5,
					max = 1.0,
					isPercent = false,
					get = function() return Numen.db.profile.confPopScale; end,
					set = function(v)
						Numen.db.profile.confPopScale = v;
						Numen:UpdateLayout();
					end
				},
				confPadding = {
					type = "range",
					name = L["Padding"],
					desc = L["Set the padding of the main Numen buttons."],
					order = 2,
					step = 1.0,
					min = 0.0,
					max = 25.0,
					isPercent = false,
					get = function() return Numen.db.profile.confPadding; end,
					set = function(v)
						Numen.db.profile.confPadding = v;
						Numen:UpdateLayout();
					end
				},
				confAlpha = {
					type = "range",
					name = L["Main Alpha"],
					desc = L["Set the alpha of the main Numen buttons."],
					order = 3,
					step = 0.05,
					min = 0.05,
					max = 1.0,
					isPercent = false,
					get = function() return Numen.db.profile.confAlpha; end,
					set = function(v)
						Numen.db.profile.confAlpha = v;
						Numen:UpdateLayout();
					end
				},
				confPopAlpha = {
					type = "range",
					name = L["PopUp Alpha"],
					desc = L["Set the alpha of the pop-up Numen buttons."],
					order = 5,
					step = 0.05,
					min = 0.05,
					max = 1.0,
					isPercent = false,
					get = function() return Numen.db.profile.confPopAlpha; end,
					set = function(v)
						Numen.db.profile.confPopAlpha = v;
						Numen:UpdateLayout();
					end
				},
			},
		},
		lock = {
			type = "toggle",
			name = L["Lock Bar"],
			desc = L["Lock the Numen Bar."],
			order = 2,
			get = function() return Numen.isLocked; end,
			set = function(v) Numen:ToggleLock(); end,
		},
	},
}

local defaults = {
	orient = "horzup",
	confScale = 1,
	confPopScale = 1,
	confPadding = 0, 
	confAlpha = 1,
	confPopAlpha = 1,
	totemStates = {},
	totemOrder = {
		earth = {},
		fire = {},
		water = {},
		air = {},
		weapon = {},
	},
}

 -- {{{ FuBar settings
Numen.name                   = "Numen"
Numen.hasIcon                = "Interface\\Icons\\Spell_Fire_TotemOfWrath"
Numen.defaultMinimapPosition = 240
Numen.cannotDetachTooltip    = true
Numen.independentProfile     = true
Numen.defaultPosition        = "LEFT"
Numen.hideWithoutStandby     = true

 -- }}}

function Numen:OnInitialize()
    -- Called when the addon is loaded
	self:RegisterDB("NumenDB", "NumenDBPC")
	self:RegisterDefaults("profile", defaults)
	self:RegisterChatCommand("/numen", self.options)

	self.keyframe = CreateFrame("Frame", nil, UIParent);
	self:RegisterEvent("PLAYER_REGEN_DISABLED", "EnterCombat")
	self:RegisterEvent("PLAYER_REGEN_ENABLED", "LeaveCombat")
	self:RegisterEvent("UPDATE_BINDINGS")
	BINDING_HEADER_Numen = "Numen";
	
	Numen.config = self.db.profile;
	self.OnMenuRequest = self.options;
end

function Numen:OnEnable()
	local _, class = UnitClass("player");
	if (class ~= "SHAMAN") then 
		DisableAddOn("Numen");
		return;
	end	
	Numen.loaded = false;
	self:SetDebugging(true);
	
	self:UpdateSpells();
	self:InitRoot();
	self:LoadPosition();
	self:UpdateLayout();
	self:RegisterOverrideBindings();
	Numen.loaded = true;
--[[	
	self:SetTotemOrder();
--]]
end

function Numen:OnDisable()
    -- Called when the addon is disabled
end

function Numen:InitRoot()
	self.root = CreateFrame("Frame", "NumenRoot", UIParent, "SecureStateHeaderTemplate");
	self.root:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	self.root:RegisterForDrag("LeftButton");
	self.root:EnableMouse(false);
	self.root:SetMovable(true);
	self.root.texture = self.root:CreateTexture();
	self.root.texture:SetTexture(0, 0, 0.5, 0);
	self.root.texture:SetAllPoints(self.root);
	self.root:SetClampedToScreen(true);
	
	-- Initialize Bars
	self.root.buttons = {};
	for type, args in pairs(self.totems) do
		self.root.buttons[args.index] = Numen.Button:new(args.index, type, args.order);
	end
end

function Numen:UPDATE_BINDINGS()
	if (not InCombatLockdown()) then
		self:RegisterOverrideBindings();
	end
end

function Numen:RegisterOverrideBindings()
	ClearOverrideBindings(self.keyframe)
	for totemType, args in pairs(self.totems) do
		local key1, key2 = GetBindingKey("Numen"..totemType.."Button");
		if key1 then
			SetOverrideBindingClick(self.keyframe, false, key1, "Numen"..totemType.."Button");
		end
		if key2 then
			SetOverrideBindingClick(self.keyframe, false, key2, "Numen"..totemType.."Button");
		end
	end
end

function Numen:OnTooltipUpdate()
	Tablet:SetHint("|cffeda55fLeft Click|r to lock Numen bar.\n|cffeda55fRight-Click|r to open config.")
end

function Numen:OnClick(button)
	-- function for fubar left click
	self:ToggleLock();
--	self:UpdateDisplay();
end

function Numen:UpdateLayout()
	local rootx, rooty = 0, 0;
	local isVert, isRtDn = false, false;
	if (self.config.orient == "horzdown") then
		isRtDn = true;
	elseif (self.config.orient == "vertleft") then
		isVert = true;
	elseif (self.config.orient == "vertright") then
		isVert = true;
		isRtDn = true;
	end
	if (isVert) then
		rootx = 36;
	else
		rooty = 36;
	end
	for idx, buttonClass in pairs(self.root.buttons) do
		local x, y = 0, 0;
		local padding = Numen.db.profile.confPadding;
		if (isVert) then
			y = (-36-padding)*(buttonClass.order - 1);
			rooty = rooty + 36 + padding;
		else
			x = (36+padding)*(buttonClass.order - 1);
			rootx = rootx + 36 + padding;
		end
		buttonClass:UpdateLayout(x, y, isVert, isRtDn);
	end
	self.root:SetWidth(rootx);
	self.root:SetHeight(rooty);
end

function Numen:ShowTooltip(button)
	if (button.spellID) then
		GameTooltip:SetOwner(button);
		GameTooltip:SetSpell(button.spellID, BOOKTYPE_SPELL);
	end
end

function Numen:HideTooltip()
	GameTooltip:Hide();
end

function Numen:OnClickPopButton(popHandle, button, down)
	popButton = popHandle:GetParent();
	self:Debug("I clicked on "..popButton:GetName());
end

function Numen:ToggleLock()
	if (self.isLocked) then
		self.root:EnableMouse(true);
		self.root:SetScript("OnDragStart", function() Numen:StartDrag(); end);
		self.root:SetScript("OnDragStop", function() Numen:StopDrag(); end);
		self.root.texture:SetTexture(0, 0, 0.5, 0.5)
		self.root:SetFrameLevel(4);
		self.isLocked = false;
		-- Show all popups
		for idx, buttonClass in pairs(self.root.buttons) do
			buttonClass:ShowPops();
		end
	else
		self.root:EnableMouse(false);
		self.root:SetScript("OnDragStart", nil);
		self.root:SetScript("OnDragStop", nil);
		self.root.texture:SetTexture(0, 0, 0.5, 0);
		self.isLocked = true;
		-- Hide all popups
		self.root:SetAttribute("state", 0);
	end		
end

function Numen:StartDrag()
	Numen.root:StartMoving();
end

function Numen:StopDrag()
	Numen.root:StopMovingOrSizing();
	Numen:SavePosition();
	
end

function Numen:LoadPosition()
	local x, y, s = self.config.rootX, self.config.rootY, self.root:GetEffectiveScale();
	if x and y then 
		x, y = x/s, y/s;
	end
	self.root:ClearAllPoints();
	self.root:SetPoint(x and "TOPLEFT" or "CENTER", UIParent, x and "BOTTOMLEFT" or "CENTER", x or 0, y or 0);
end

function Numen:SavePosition()
	local x, y, s = self.root:GetLeft(), self.root:GetTop(), self.root:GetEffectiveScale();
	x, y = x*s, y*s;
	self.config.rootX = x;
	self.config.rootY = y;
end

function Numen:UpdateSpells()
	local spellID = 1;
	repeat
		local spellName, _ = GetSpellName(spellID, BOOKTYPE_SPELL);
		if (self.spells[spellName]) then
			local spellType = self.spells[spellName].type;
			local gIndex = self.spells[spellName].gIndex;
			
			local spell = self.totems[spellType].spells[spellName];
			if (spell) then
				-- Compare SpellID with what we currently have and make sure this one is higher.
				if (spell.spellID < spellID) then
					-- It's higher, let's replace it
					self.totems[spellType].spells[spellName].spellID = spellID;
				end
			else
				-- Not in our table yet, let's added it!
				self.totems[spellType].spells[spellName] = {};
				self.totems[spellType].spells[spellName].gIndex = gIndex;
				self.totems[spellType].spells[spellName].spellID = spellID;
				self.totems[spellType].spellCount = self.totems[spellType].spellCount + 1;
				self.totems[spellType].spells[spellName].order = self.totems[spellType].spellCount;
			end
		end
		spellID = spellID + 1;
	until (not spellName);
end

------------------------------------------
--  Debugging Functions			--
------------------------------------------
--[[
function Numen:SetTotemOrder()
	-- For now atleast...
	-- Assign the order based on what's saved, if it exists, otherwise remove it from saved list
	for totemType, typeArgs in pairs(self.config.totemOrder) do
		maxOrder = 0;
		for totemOrder, totemArgs in pairs(typeArgs) do
			if (self.totems[totemType].totems[totemArgs.name] and not self.totems[totemType].totems[totemArgs.name].order) then
				self.totems[totemType].totems[totemArgs.name].order = tonumber(totemOrder);
				if (tonumber(totemOrder) > maxOrder) then
					maxOrder = tonumber(totemOrder);
				end
			else
				Numen:Debug(totemArgs.name.." doesn't exist");
				self.config.totemOrder[totemType][totemOrder] = nil;
			end
		end
		-- Add unassigned totems to the end of the list.
		for totemName, totemArgs in pairs(self.totems[totemType].totems) do
			if (not totemArgs.order) then
				maxOrder = maxOrder + 1;
				totemArgs.order = maxOrder;
				typeArgs[tostring(maxOrder)] = {name = totemName};
			end
		end
		-- Reorder the list to remove blank spots.
		idx = 1
		while (idx < maxOrder) do
			grabidx = idx + 1;
			while (not typeArgs[tostring(idx)] and grabidx <= maxOrder) do
				if (not typeArgs[tostring(grabidx)]) then
					grabidx = grabidx + 1;
				else
					self.totems[totemType].totems[typeArgs[tostring(grabidx)].name].order = idx;
					typeArgs[tostring(idx)] = typeArgs[tostring(grabidx)];
					typeArgs[tostring(grabidx)] = nil;
				end
			end
			idx = idx + 1;
		end
	end
end
--]]

-- ----------------------- --
-- New Code                --
-- ----------------------- --

--[[ Party Indicators Base Code
function Numen:AddIndicators(numenPartyIndex)
	-- for loop, 1-4 (or 0-3) for totem types (typeIndex) 
		-- popIndicator[numenPartyIndex][type/main] ok?, scope ok? replace anchor place holders, just put on popup button 1 to start
		mainIndicator[numenPartyIndex][typeIndex] = CreateFrame("Frame", "", ****MAINBUTTON ANCHOR****);
		mainIndicator[numenPartyIndex][typeIndex]:SetWidth(8);
		mainIndicator[numenPartyIndex][typeIndex]:SetHeight(8);
		mainIndicator[numenPartyIndex][typeIndex]:SetBackdrop( {
			      bgFile = "Interface\\Addons\\Numen\\white16x16", tile = true, tileSize = 16,
			      edgeFile = "Interface\\Addons\\Numen\\white16x16", edgeSize = 1,
			      insets = {left = 1, right = 1, top = 1, bottom = 1},
		      });
		mainIndicator[numenPartyIndex][typeIndex]:SetBackdropBorderColor(0,0,0,1);
		mainIndicator[numenPartyIndex][typeIndex]:SetBackdropColor(1,1,1,1);
		mainIndicator[numenPartyIndex][typeIndex]:SetFrameLevel(5);
		-- separator for main/pop indicators
		popIndicator[numenPartyIndex][typeIndex] = CreateFrame("Frame", "", ****POP UP BUTTON ANCHOR****);
		popIndicator[numenPartyIndex][typeIndex]:SetWidth(8);
		popIndicator[numenPartyIndex][typeIndex]:SetHeight(8);
		popIndicator[numenPartyIndex][typeIndex]:SetBackdrop( {
			      bgFile = "Interface\\Addons\\Numen\\white16x16", tile = true, tileSize = 16,
			      edgeFile = "Interface\\Addons\\Numen\\white16x16", edgeSize = 1,
			      insets = {left = 1, right = 1, top = 1, bottom = 1},
		      });
		popIndicator[numenPartyIndex][typeIndex]:SetBackdropBorderColor(0,0,0,1);
		popIndicator[numenPartyIndex][typeIndex]:SetBackdropColor(1,1,1,1);
		popIndicator[numenPartyIndex][typeIndex]:SetFrameLevel(5);
		if numenPartyIndex == 3 then
			popIndicator[numenPartyIndex][typeIndex]:SetPoint("BOTTOMLEFT", ****POP UP BUTTON ANCHOR****, "BOTTOMLEFT", 1, 1)
			mainIndicator[numenPartyIndex][typeIndex]:SetPoint("BOTTOMLEFT", ****MAINBUTTON ANCHOR****, "BOTTOMLEFT", 1, 1)
		elseif numenPartyIndex == 2 then
			popIndicator[numenPartyIndex][typeIndex]:SetPoint("BOTTOMRIGHT", ****POP UP BUTTON ANCHOR****, "BOTTOMRIGHT", -1, 1)
			mainIndicator[numenPartyIndex][typeIndex]:SetPoint("BOTTOMRIGHT", ****MAINBUTTON ANCHOR****, "BOTTOMRIGHT", -1, 1)
		elseif numenPartyIndex == 1 then
			popIndicator[numenPartyIndex][typeIndex]:SetPoint("TOPRIGHT", ****POP UP BUTTON ANCHOR****, "TOPRIGHT", -1, -1)
			mainIndicator[numenPartyIndex][typeIndex]:SetPoint("TOPRIGHT", ****MAINBUTTON ANCHOR****, "TOPRIGHT", -1, -1)
		elseif numenPartyIndex == 0 then
			popIndicator[numenPartyIndex][typeIndex]:SetPoint("TOPLEFT", ****POP UP BUTTON ANCHOR****, "TOPLEFT", 1, -1)
			mainIndicator[numenPartyIndex][typeIndex]:SetPoint("TOPLEFT", ****MAINBUTTON ANCHOR****, "TOPLEFT", 1, -1)
		else
			Numen:Debug("ERROR, numenPartyIndex in AddIndicators is not 0-3!")
		end
		popIndicator[numenPartyIndex][typeIndex]:Hide();
		mainIndicator[numenPartyIndex][typeIndex]:Hide();
	--end for loop
end
function Numen:MoveIndicator(spellName, typeIndex, numenPartyIndex)
	--not sure on naming and scope, depends on how you do AddIndicators
	popIndicator[numenPartyIndex][typeIndex]:ClearAllPoints();
	--change anchor place holders to correct location that corresponds to spellName
	if numenPartyIndex == 3 then
		popIndicator:SetPoint("BOTTOMLEFT", ****POP UP BUTTON ANCHOR****, "BOTTOMLEFT", 1, 1)
	elseif numenPartyIndex == 2 then
		popIndicator:SetPoint("BOTTOMRIGHT", ****POP UP BUTTON ANCHOR****, "BOTTOMRIGHT", -1, 1)
	elseif numenPartyIndex == 1 then
		popIndicator:SetPoint("TOPRIGHT", ****POP UP BUTTON ANCHOR****, "TOPRIGHT", -1, -1)
	elseif numenPartyIndex == 0 then
		popIndicator:SetPoint("TOPLEFT", ****POP UP BUTTON ANCHOR****, "TOPLEFT", 1, -1)
	else
		Numen:Debug("ERROR, numenPartyIndex in MoveIndicator is not 0-3!")
	end
	popIndicator[numenPartyIndex][typeIndex]:Show();
	-- if popIndicator[numenPartyIndex][typeIndex] is same as main button spell, mainIndicator[numenPartyIndex][typeIndex]:Show()
end

function checkMainIndicator()
	--for loop to check main indicator vs pop up indicators
end

function Numen:sendAddonMsg(spellGIndex, typeIndex)
	-- spell gindex used, type must be 0-3 based on earth, fire, air, water
	SendAddonMessage("NumenMsg", "up("..typeIndex..")"..spellGIndex, "PARTY");
end

function Numen:recAddonMsg()
	--arg1 - arg4 are sent: header, message, way sent - party etc, player name
	if (arg1 == "NumenMsg") then
		-- if arg4 (player name) is NOT in database for players
			-- add arg4 (player name) to party numen users database
			-- if mainIndicator[playerIndex][0] doesn't exist, (if frame isn't already made)
				-- self:AddIndicators(playerIndex);
			-- end
		-- end
		-- pIndex = party member index of player that sent message 
		if (string.find(arg2, "up(")) then
			local spellChanged = string.sub(arg2, 6);
			local typeChanged = string.sub(arg2, 4, 4);
			self:MoveIndicator(spellChanged, typeChanged, pIndex);
		end
	end
end

function Numen:PartyChanged()
	--reset party table of people using numen
	--hide all indicators
	--for loop to send all 4 set totems
end


-- ********event registers************
self:RegisterEvent("PARTY_MEMBERS_CHANGED", "PartyChanged")
self:RegisterEvent("CHAT_MSG_ADDON", "RecAddonMsg")

--]]

function Numen:UpdateMacro()
	if (not Numen.loaded) then return end;
	local numenMacroID = GetMacroIndexByName("NumenStomper");
	if InCombatLockdown() then
		self.dirtyMacro = true;
	else	
		if (numenMacroID == 0) then
			numenMacroID = CreateMacro("NumenStomper", 762, "", 1, 1);
		end
		local totemMacroList = "/castsequence reset=combat/shift "
		for idx = 1, 4 do
			spellID = Numen.root.buttons[idx].button.spellID;
			totemMacroList = totemMacroList..GetSpellName(spellID, BOOKTYPE_SPELL);
			if (idx < 4) then
				totemMacroList = totemMacroList..", ";
			end
		end
		totemMacroList = totemMacroList..";";
--		Numen:Debug(totemMacroList);		
		EditMacro(numenMacroID, "NumenStomper", 762, totemMacroList, 1);
	end
end

function Numen:EnterCombat()
	-- if bars unlocked, lock bars
end

function Numen:LeaveCombat()
	if (self.dirtyMacro) then
		self.dirtyMacro = false
		self:updateMacro()
	end
end



